/* vim: set ro : */

/*
 * libsyd: syd /dev/syd interface implementation
 * Copyright (c) 2023, 2024 Ali Polatel <alip@chesswob.org>
 * SPDX-License-Identifier: LGPL-3.0-or-later
 */


#ifndef LIBSYD_3_SYD_H
#define LIBSYD_3_SYD_H

/* THIS IS A GENERATED FILE. DO NOT EDIT THIS FILE DIRECTLY */


#include <stdarg.h>
#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>
#include <stdlib.h>


/**
 * An enumeration of the possible actions for force sandboxing.
 */
enum force_action_t {
  /**
   * Allow system call.
   */
  FORCE_ALLOW,
  /**
   * Print a warning to syslog.
   */
  FORCE_WARN,
  /**
   * Kill process with SIGKILL.
   */
  FORCE_KILL,
};
typedef uint8_t force_action_t;

/**
 * An enumeration of the possible states for the sandbox lock.
 */
enum lock_state_t {
  /**
   * The sandbox lock is off, allowing all sandbox commands.
   */
  LOCK_OFF,
  /**
   * The sandbox lock is set to on for all processes except the initial
   * process (syd exec child). This is the default state.
   */
  LOCK_EXEC,
  /**
   * The sandbox lock is on, disallowing all sandbox commands.
   */
  LOCK_ON,
};
typedef uint8_t lock_state_t;

/**
 * Adds to the allowlist of exec sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_allow_exec_add(const char *glob);

/**
 * Removes the first instance from the end of the allowlist of exec
 * sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_allow_exec_del(const char *glob);

/**
 * Removes all matching patterns from the allowlist of exec sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_allow_exec_rem(const char *glob);

/**
 * Adds to the allowlist of net/bind sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_allow_net_bind_add(const char *addr);

/**
 * Removes the first instance from the end of the allowlist of net/bind
 * sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_allow_net_bind_del(const char *addr);

/**
 * Removes all matching patterns from the allowlist of net/bind sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_allow_net_bind_rem(const char *addr);

/**
 * Adds to the allowlist of net/connect sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_allow_net_connect_add(const char *addr);

/**
 * Removes the first instance from the end of the allowlist of net/connect
 * sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_allow_net_connect_del(const char *addr);

/**
 * Removes all matching patterns from the allowlist of net/connect sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_allow_net_connect_rem(const char *addr);

/**
 * Adds to the allowlist of read sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_allow_read_add(const char *glob);

/**
 * Removes the first instance from the end of the allowlist of read
 * sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_allow_read_del(const char *glob);

/**
 * Removes all matching patterns from the allowlist of read sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_allow_read_rem(const char *glob);

/**
 * Adds to the allowlist of stat sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_allow_stat_add(const char *glob);

/**
 * Removes the first instance from the end of the allowlist of stat
 * sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_allow_stat_del(const char *glob);

/**
 * Removes all matching patterns from the allowlist of stat sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_allow_stat_rem(const char *glob);

/**
 * Adds to the allowlist of write sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_allow_write_add(const char *glob);

/**
 * Removes the first instance from the end of the allowlist of write
 * sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_allow_write_del(const char *glob);

/**
 * Removes all matching patterns from the allowlist of write sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_allow_write_rem(const char *glob);

/**
 * Performs a syd API check
 *
 * The caller is advised to perform this check before
 * calling any other syd API calls.
 *
 * Returns API number on success, negated errno on failure.
 */
int syd_api(void);

/**
 * Performs an lstat system call on the file "/dev/syd".
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_check(void);

/**
 * Adds to the denylist of exec sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_deny_exec_add(const char *glob);

/**
 * Removes the first instance from the end of the denylist of exec
 * sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_deny_exec_del(const char *glob);

/**
 * Removes all matching patterns from the denylist of exec sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_deny_exec_rem(const char *glob);

/**
 * Adds to the denylist of net/bind sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_deny_net_bind_add(const char *addr);

/**
 * Removes the first instance from the end of the denylist of net/bind
 * sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_deny_net_bind_del(const char *addr);

/**
 * Removes all matching patterns from the denylist of net/bind sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_deny_net_bind_rem(const char *addr);

/**
 * Adds to the denylist of net/connect sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_deny_net_connect_add(const char *addr);

/**
 * Removes the first instance from the end of the denylist of net/connect
 * sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_deny_net_connect_del(const char *addr);

/**
 * Removes all matching patterns from the denylist of net/connect sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_deny_net_connect_rem(const char *addr);

/**
 * Adds to the denylist of read sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_deny_read_add(const char *glob);

/**
 * Removes the first instance from the end of the denylist of read
 * sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_deny_read_del(const char *glob);

/**
 * Removes all matching patterns from the denylist of read sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_deny_read_rem(const char *glob);

/**
 * Adds to the denylist of stat sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_deny_stat_add(const char *glob);

/**
 * Removes the first instance from the end of the denylist of stat
 * sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_deny_stat_del(const char *glob);

/**
 * Removes all matching patterns from the denylist of stat sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_deny_stat_rem(const char *glob);

/**
 * Adds to the denylist of write sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_deny_write_add(const char *glob);

/**
 * Removes the first instance from the end of the denylist of write
 * sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_deny_write_del(const char *glob);

/**
 * Removes all matching patterns from the denylist of write sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_deny_write_rem(const char *glob);

/**
 * Disable exec sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_disable_exec(void);

/**
 * Disable force sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_disable_force(void);

/**
 * Disable memory sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_disable_mem(void);

/**
 * Disable net sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_disable_net(void);

/**
 * Disable PID sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_disable_pid(void);

/**
 * Disable read sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_disable_read(void);

/**
 * Disable stat sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_disable_stat(void);

/**
 * Disable write sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_disable_write(void);

/**
 * Enable exec sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_enable_exec(void);

/**
 * Enable force sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_enable_force(void);

/**
 * Enable memory sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_enable_mem(void);

/**
 * Enable net sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_enable_net(void);

/**
 * Enable PID sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_enable_pid(void);

/**
 * Enable read sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_enable_read(void);

/**
 * Enable stat sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_enable_stat(void);

/**
 * Enable write sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_enable_write(void);

/**
 * Checks if exec sandboxing is enabled.
 *
 * Returns true if exec sandboxing is enabled, false otherwise.
 */
bool syd_enabled_exec(void);

/**
 * Checks if force sandboxing is enabled.
 *
 * Returns true if force sandboxing is enabled, false otherwise.
 */
bool syd_enabled_force(void);

/**
 * Checks if memory sandboxing is enabled.
 *
 * Returns true if memory sandboxing is enabled, false otherwise.
 */
bool syd_enabled_mem(void);

/**
 * Checks if net sandboxing is enabled.
 *
 * Returns true if net sandboxing is enabled, false otherwise.
 */
bool syd_enabled_net(void);

/**
 * Checks if PID sandboxing is enabled.
 *
 * Returns true if PID sandboxing is enabled, false otherwise.
 */
bool syd_enabled_pid(void);

/**
 * Checks if read sandboxing is enabled.
 *
 * Returns true if read sandboxing is enabled, false otherwise.
 */
bool syd_enabled_read(void);

/**
 * Checks if stat sandboxing is enabled.
 *
 * Returns true if stat sandboxing is enabled, false otherwise.
 */
bool syd_enabled_stat(void);

/**
 * Checks if write sandboxing is enabled.
 *
 * Returns true if write sandboxing is enabled, false otherwise.
 */
bool syd_enabled_write(void);

/**
 * Execute a command outside the sandbox without sandboxing
 *
 * # Safety
 *
 * This function is marked `unsafe` because it dereferences raw
 * pointers, which is inherently unsafe in Rust.
 *
 * The caller must ensure the following conditions are met to safely
 * use this function:
 *
 * 1. The `file` pointer must point to a valid, null-terminated C-style
 *    string.
 *
 * 2. The `argv` pointer must point to an array of pointers, where each
 *    pointer refers to a valid, null-terminated C-style string. The
 *    last pointer in the array must be null, indicating the end of the
 *    array.
 *
 * 3. The memory pointed to by `file` and `argv` must remain valid for
 *    the duration of the call.
 *
 * Failing to uphold these guarantees can lead to undefined behavior,
 * including memory corruption and data races.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_exec(const char *file, const char *const *argv);

/**
 * Adds to the filter of exec sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_filter_exec_add(const char *glob);

/**
 * Removes the first instance from the end of the filter of exec
 * sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_filter_exec_del(const char *glob);

/**
 * Removes all matching patterns from the filter of exec sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_filter_exec_rem(const char *glob);

/**
 * Toggle the reporting of access violations for memory sandboxing
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_filter_mem(bool state);

/**
 * Adds to the filter of net/bind sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_filter_net_bind_add(const char *addr);

/**
 * Removes the first instance from the end of the filter of net/bind
 * sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_filter_net_bind_del(const char *addr);

/**
 * Removes all matching patterns from the filter of net/bind sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_filter_net_bind_rem(const char *addr);

/**
 * Adds to the filter of net/connect sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_filter_net_connect_add(const char *addr);

/**
 * Removes the first instance from the end of the filter of net/connect
 * sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_filter_net_connect_del(const char *addr);

/**
 * Removes all matching patterns from the filter of net/connect sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_filter_net_connect_rem(const char *addr);

/**
 * Toggle the reporting of access violations for PID sandboxing
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_filter_pid(bool state);

/**
 * Adds to the filter of read sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_filter_read_add(const char *glob);

/**
 * Removes the first instance from the end of the filter of read
 * sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_filter_read_del(const char *glob);

/**
 * Removes all matching patterns from the filter of read sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_filter_read_rem(const char *glob);

/**
 * Adds to the filter of stat sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_filter_stat_add(const char *glob);

/**
 * Removes the first instance from the end of the filter of stat
 * sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_filter_stat_del(const char *glob);

/**
 * Removes all matching patterns from the filter of stat sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_filter_stat_rem(const char *glob);

/**
 * Adds to the filter of write sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_filter_write_add(const char *glob);

/**
 * Removes the first instance from the end of the filter of write
 * sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_filter_write_del(const char *glob);

/**
 * Removes all matching patterns from the filter of write sandboxing.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_filter_write_rem(const char *glob);

/**
 * Adds an entry to the Integrity Force map for Force Sandboxing.
 *
 * # Safety
 *
 * This function is marked `unsafe` because it dereferences raw
 * pointers, which is inherently unsafe in Rust.
 *
 * The caller must ensure the following conditions are met to safely
 * use this function:
 *
 * 1. The `path` pointer must point to a valid, null-terminated C-style
 *    string.
 * 2. The `hash` pointer must point to a valid, null-terminated C-style
 *    string.
 */
int syd_force_add(const char *path, const char *hash, force_action_t action);

/**
 * Clears the Integrity Force map for Force Sandboxing.
 */
int syd_force_clr(void);

/**
 * Set the default action for Force Sandboxing.
 */
int syd_force_def(force_action_t action);

/**
 * Removes an entry from the Integrity Force map for Force Sandboxing.
 * # Safety
 *
 * This function is marked `unsafe` because it dereferences raw
 * pointers, which is inherently unsafe in Rust.
 *
 * The caller must ensure the following conditions are met to safely
 * use this function:
 *
 * 1. The `path` pointer must point to a valid, null-terminated C-style
 *    string.
 */
int syd_force_del(const char *path);

/**
 * Adds to the list of glob patterns used to determine which paths
 * should be killed (prevented from executing) in the sandbox.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_kill_add(const char *glob);

/**
 * Deletes the first matching item from the end of the list of glob
 * patterns used to determine which paths should be killed (prevented
 * from executing) in the sandbox.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_kill_del(const char *glob);

/**
 * Toggle kill of the offending process for Memory sandboxing
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_kill_mem(bool state);

/**
 * Toggle kill of the offending process for PID sandboxing
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_kill_pid(bool state);

/**
 * Removes all matching items from the list of glob patterns used to
 * determine which paths should be killed (prevented from executing) in
 * the sandbox.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_kill_rem(const char *glob);

/**
 * Causes syd to read configuration from the given file descriptor.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_load(int fd);

/**
 * Sets the state of the sandbox lock.
 *
 * state: The desired state of the sandbox lock.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_lock(lock_state_t state);

/**
 * Set syd maximum per-process memory usage limit for memory sandboxing.
 *
 * parse-size crate is used to parse the value so formatted strings are OK.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_mem_max(const char *size);

/**
 * Set syd maximum per-process virtual memory usage limit for memory sandboxing.
 *
 * parse-size crate is used to parse the value so formatted strings are OK.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_mem_vm_max(const char *size);

/**
 * Causes syd to exit immediately with code 127
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_panic(void);

/**
 * Set syd maximum process id limit for PID sandboxing
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_pid_max(size_t size);

/**
 * Causes syd to reset sandboxing to the default state.
 * Allowlists, denylists and filters are going to be cleared.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_reset(void);

/**
 * Specify SegvGuard entry expiry timeout in seconds.
 * Setting this timeout to 0 effectively disables SegvGuard.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_segvguard_expiry(uint64_t timeout);

/**
 * Specify SegvGuard max number of crashes before suspension.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_segvguard_maxcrashes(uint8_t max);

/**
 * Specify SegvGuard entry suspension timeout in seconds.
 *
 * Returns 0 on success, negated errno on failure.
 */
int syd_segvguard_suspension(uint64_t timeout);

#endif /* LIBSYD_3_SYD_H */
