//
// syd: seccomp and landlock based application sandbox with support for namespaces
// src/syd-cpu.rs: Print the number of CPUs.
//
// Copyright (c) 2024 Ali Polatel <alip@chesswob.org>
//
// SPDX-License-Identifier: GPL-3.0-or-later

use std::{env, process::ExitCode};

use getargs::{Opt, Options};

fn main() -> ExitCode {
    #[allow(clippy::disallowed_methods)]
    syd::set_sigpipe_dfl().expect("SIGPIPE");

    let args = env::args().skip(1).collect::<Vec<_>>();
    let mut opts = Options::new(args.iter().map(String::as_str));
    let mut physical = false;

    // SAFETY: We panic on parse errors.
    #[allow(clippy::disallowed_methods)]
    while let Some(opt) = opts.next_opt().expect("next opt") {
        match opt {
            Opt::Short('h') => {
                help();
                return ExitCode::SUCCESS;
            }
            Opt::Short('l') => {
                physical = false;
            }
            Opt::Short('p') => {
                physical = true;
            }
            _ => {
                eprintln!("Unknown option: {opt:?}!");
                return ExitCode::FAILURE;
            }
        }
    }

    let num = if physical {
        num_cpus::get_physical()
    } else {
        num_cpus::get()
    };
    println!("{num}");

    ExitCode::SUCCESS
}

fn help() {
    println!("Usage: syd-cpu [-hlp]");
    println!("Print the number of CPUs.");
    println!("Use -l to print the number of logical CPUs (default).");
    println!("Use -p to print the number of physical CPUs.");
}
